<?php

namespace Drupal\graphapi\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Fhaculty\Graph\Graph;

/**
 * Returns responses for devel module routes.
 */
class GraphAPIController extends ControllerBase {

  /**
   * Returns a list of all currently defined user functions in the current
   * request lifecycle, with links their documentation.
   */
  public function functionReference() {
    $functions = get_declared_classes();
    sort($functions);
    $links = array();
    foreach ($functions as $function) {
      $links[] = $function;
    }
    return theme('item_list', array('items' => $links));
  }

  public function getLocalTasks() {
    $configs = drupal_container()->get('config.storage')->listAll();
    return print_r($configs, TRUE);
  }

  public function formatDemo($format_id) {
    $build = [];

    $format = \Drupal::service('plugin.manager.graphapi_graph_format')->createInstance($format_id);

    $build['#title'] = $this->t("Demo for @engine graph format", [
      '@engine' => $format->getLabel(),
    ]);

    $graph = $this->demoGraph();

    $build['graph'] = [
      '#type' => 'graph',
      '#graph' => $graph,
      '#format' => $format_id,
      '#width' => 800,
      '#height' => 800,
    ];

    return $build;
  }

  public function format_form($format) {
    $formats = $this->getFormats();
    if (key_exists($format, $formats)) {
      drupal_set_title("Form for format $format");
      return \Drupal::formBuilder()->getForm('graphapi_engine_form', $format);
    }
    else {
      return "Error";
    }
  }

  /**
   * Get all modules implementing graphapi_formats
   *
   * @todo: remove this and update callers to use plugin definitions.
   * @return type
   */
  private function getFormats() {
    $engines = \Drupal::service('plugin.manager.graphapi_graph_format')->getDefinitions();
    return array_column($engines, 'label', 'id');
  }

  function getImplementations() {
    return array(
      'graphapi_formats',
    );
  }

  /**
   * Creates a simple demo graph.
   *
   * @return \Fhaculty\Graph\Graph
   *   The graph.
   */
  public function demoGraph(): Graph {
    $graph = new Graph();

    $tags = $graph->createVertex('tags');
    $tags->setAttribute('title', 'Tags');

    $user = $graph->createVertex('user');
    $user->setAttribute('title', 'Users');

    $article = $graph->createVertex('article');
    $article->setAttribute('title', 'Article');

    $article->createEdgeTo($tags)->setAttribute('title', 'field_tags');
    $article->createEdgeTo($user)->setAttribute('title', 'uid');

    $page = $graph->createVertex('page');
    $page->setAttribute('title', 'Page');

    $page->createEdgeTo($user)->setAttribute('title', 'uid');

    return $graph;
  }

}
