<?php

namespace Drupal\graphapi\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Fhaculty\Graph\Graph;

/**
 * Displays all graph API formats and engines in a table.
 */
class GraphFormatsListController extends ControllerBase {

  /**
   * Route callback to display a table of engines.
   */
  public function build() {
    $build = [];

    $headers = [
      $this->t('Format'),
      $this->t('Engine'),
      $this->t('Description'),
      [
        'data' => $this->t('Operations'),
        'colspan' => 7,
      ],
    ];

    $formats = \Drupal::service('plugin.manager.graphapi_graph_format')->getDefinitions();
    $engines = \Drupal::service('plugin.manager.graphapi_graph_engine')->getDefinitions();

    if (empty($formats)) {
      return [
        '#markup' => $this->t('No Graph API engines found.'),
      ];
    }

    uasort($formats, function ($definition_a, $definition_b) {
      return strnatcasecmp($definition_a['label'], $definition_b['label']);
  });

    $rows = [];
    foreach ($formats as $format_id => $format_definition) {
      $row = [
        $format_id,
        $format_definition['engine'],
        $format_definition['label'],
      ];

      $operations = [
        'demo' => [
          'title' => $this->t('Demo'),
          'url' => Url::fromRoute('graphapi.format.demo', ['format_id' => $format_id]),
        ],
        'form' => [
          'title' => $this->t('Form'),
          'url' => Url::fromRoute('graphapi.engine_form', ['format' => $format_id]),
        ],
        'format_defaults' => [
          'title' => $this->t('Configure format defaults'),
          'url' => Url::fromRoute('graphapi.format.default_options', ['format_id' => $format_id]),
        ],
      ];

      if ($engines[$format_definition['engine']]['configurable']) {
        $operations['configure_engine'] = [
          'title' => $this->t('Configure engine'),
          'url' => Url::fromRoute('graphapi.engine.settings', ['engine_id' => $format_definition['engine']]),
        ];
      }

      $row[] = [
        'data' => [
          '#type' => 'operations',
          '#links' => $operations,
        ],
      ];
      $rows[] = $row;
    }

    $build['table'] = [
      '#type' => 'table',
      '#header' => $headers,
      '#rows' => $rows,
    ];

    return $build;
  }

}
